
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#pragma once

#include <glib-object.h>

G_BEGIN_DECLS
/* enumerations from "/build/budgie-control-center/src/build/../budgie-control-center/panels/common/gsd-device-manager.h" */
GType gsd_device_type_get_type (void) G_GNUC_CONST;
#define GSD_TYPE_DEVICE_TYPE (gsd_device_type_get_type())
G_END_DECLS

/* Generated data ends here */

